/* res_messaging.c 
 *
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Copyright (C) 2005, Joshua Colp
 *
 * Joshua Colp, <joshnet@nbnet.nb.ca>
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "asterisk.h"

#include "asterisk/lock.h"
#include "asterisk/channel.h"
#include "asterisk/logger.h"
#include "asterisk/file.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/manager.h"
#include "asterisk/cli.h"
#include "asterisk/messaging.h"
#include "asterisk/app.h"
#include "asterisk/utils.h"
#include "asterisk/config.h"
#include "asterisk/astobj.h"

struct conversation {
	ASTOBJ_COMPONENTS(struct conversation);		
	char masquerade[32];			/* Who we do we masquerade as when talking to this conversation? */
	int status;				/* Status of conversation */
	struct ast_channel *owner;		/* Owner channel of conversation */
};

static struct ast_conversation_list {
	ASTOBJ_CONTAINER_COMPONENTS(struct conversation);
} conversations;

/* Test application */
static int text_exec(struct ast_channel *chan, void *data)
{
	int res = 0;
	char *text = NULL;
	if (chan->content != 1) /* Not carrying text... 'tsk 'tsk */
		return -1;
	text = ast_recvtext(chan, 3600);
	if (!text)
		return -1;
	ast_log(LOG_NOTICE, "Text received: %s", text);
	ast_sendtext(chan, "How are you doing?");
	free(text);
	return res;
}

static void conversation_destroy(struct conversation *conversation)
{
	conversation->owner = NULL;
	free(conversation);
	return;
}

int ast_messaging_change_masquerade(char *uniqueid, char *masquerade)
{
	struct conversation *conversation = NULL;
	int res = 0;
        conversation = ASTOBJ_CONTAINER_FIND(&conversations, uniqueid);
        if (conversation) {
		ast_log(LOG_NOTICE, "When talking to %s we will act is if we're %s\n", uniqueid, masquerade);
                strncpy(conversation->masquerade, masquerade, sizeof(conversation->masquerade));
        } else
                res = -1;
	return res;
}

int ast_messaging_add_conversation(char *uniqueid, struct ast_channel *chan)
{
	struct conversation *conversation = NULL;
	int res = 0;

	conversation = (struct conversation *)malloc(sizeof(struct conversation));
	if (!conversation)
		return -1;
	memset(conversation, 0, sizeof(struct conversation));
	/* Initialize it... */
	ASTOBJ_INIT(conversation);
	ast_copy_string(conversation->name, uniqueid, sizeof(conversation->name));
	conversation->status = 0;
	conversation->owner = chan;
	/* Now link it up */
	ASTOBJ_CONTAINER_LINK(&conversations, conversation);
	ASTOBJ_UNREF(conversation, conversation_destroy);

	return res;
}

struct ast_channel *ast_messaging_grab_conversation(char *uniqueid)
{
	struct conversation *conversation = NULL;
	conversation = ASTOBJ_CONTAINER_FIND(&conversations, uniqueid);
	if (conversation)
		return conversation->owner;
	else
		return NULL;
}

char *ast_messaging_get_masquerade(struct ast_channel *owner)
{
	char *masquerade = NULL;

	ASTOBJ_CONTAINER_TRAVERSE(&conversations, 1, do {
		ASTOBJ_RDLOCK(iterator);
		if (iterator->owner == owner)
			masquerade = iterator->masquerade;
		ASTOBJ_UNLOCK(iterator);
	} while (0) );

	return masquerade;
}

int ast_messaging_destroy_conversation(char *uniqueid)
{
	struct conversation *conversation = NULL;
	int res = 0;
	conversation = ASTOBJ_CONTAINER_FIND_UNLINK(&conversations, uniqueid);
	if (!conversation)
		return -1;
	ASTOBJ_UNREF(conversation, conversation_destroy);
	return res;
}

int load_module(void)
{
	ASTOBJ_CONTAINER_INIT(&conversations);
	ast_register_application("TextTest", text_exec, "Bleh", "Blah");
	return 0;
}

int unload_module(void)
{
	return 0;
}

char *description(void)
{
	return "Text Messaging Resource";
}

int usecount(void)
{
	return 1;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
